; **********************************************************************

; Programme de Hack du CD-723
; Intercepte la commande entre la platine et le drive pour le format de sortie :
; IS, Sony (16bits), Sony (18bits)
; Surchantillonnage 1x,2x,4x

; **********************************************************************

.include "1200def.inc"

; Definition des registres

.def	temp		= r16	; Registre "kiftou"
.def	donnee		= r17	; Une donne
.def	donneeb		= r18	; Une autre donne
.def	octet		= r19	; Nb octet  rcuprer en cas d'criture
.def	rehack		= r20	; Test de reconnaissance de dbut de lecture
.def 	hack_dac	= r21	; Nouvelle valeur pour le dac

; Dfinition des quivalences

.equ 	sicl 	= 3
.equ 	sicd 	= 4
.equ	data	= 5

; **********************************************************************

Reset:
	
; Initialisation des entres-sorties
; Leds          = PB0 ; sorties
; Config 1		= PB1 ; entre pull-up
; Config 2		= PB2 ; entre pull-up
; Config 3		= PB3 ; entre pull-up
; Config 4		= PB4 ; entre pull-up

	Ldi	temp,0b00000001
	Out	DDRB,temp
	Ldi	temp,0b11111110
	Out	PORTB,temp
	Ldi	temp,0b00000000
	Out	DDRD,temp
	Ldi	temp,0b01000111
	Out	PORTD,temp

; Programmation du Watchdog
	Wdr
	Ldi temp,0b00001011		; environ 100ms avant dclanchement
	Out WDTCR,temp

; Initialisation des variables
	Clr	rehack

; Lecture de la configuration

; 0b00111110 IS 16/18bits 1xFS
; 0b00111111 IS 18bits    2xFS
; 0b00111100 IS 18bits    4xFS
; 0b00110110 Sony 18 bits  1xFS
; 0b00110111 Sony 18 bits  2xFS
; 0b00110100 Sony 18 bits  4xFS
; 0b00110010 Sony 16 bits  1xFS
; 0b00110011 Sony 16 bits  2xFS
; 0b00110000 Sony 16 bits  4xFS

	Ldi hack_dac,0b00000010		; 1xFS par dfaut
	
	Sbis PINB,1
	Ldi hack_dac,0b00000011		; 2xFS si PB1=0
	Sbis PINB,2
	Ldi hack_dac,0b00000000		; 4xFS si PB2=0

	Ldi temp,0b00110000			; Sony 16 bits par dfaut
	Sbis PINB,3
	Ldi temp,0b00110100			; Sony 18 bits si PB3=0
	Sbis PINB,4
	Ldi temp,0b00111100			; IS si PB4=0

	Or hack_dac,temp

; **********************************************************************
; Reception de la commande

commande_0:
	Clr donnee

commande_1:					; bit 7
	Wdr						; Reset du watchdog
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp commande_1
commande_1b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp commande_1b
commande_1c:
	Sbic PIND,data			; On met  jour donnee en fonction de DATA
	Sbr donnee,0b10000000

commande_2:					; bit 6
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp commande_2
commande_2b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp commande_2b
	Sbic PIND,data			; On met  jour donnee en fonction de DATA
	Sbr donnee,0b01000000

commande_3:					; bit 5
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp commande_3
commande_3b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp commande_3b
	Sbic PIND,data			; On met  jour donnee en fonction de DATA
	Sbr donnee,0b00100000

commande_4:					; bit 4
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp commande_4
commande_4b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp commande_4b
	Sbic PIND,data			; On met  jour donnee en fonction de DATA
	Sbr donnee,0b00010000

commande_5:					; bit 3
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp commande_5
commande_5b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp commande_5b
	Sbic PIND,data			; On met  jour donnee en fonction de DATA
	Sbr donnee,0b00001000

commande_6:					; bit 2
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp commande_6
commande_6b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp commande_6b
	Sbic PIND,data			; On met  jour donnee en fonction de DATA
	Sbr donnee,0b00000100

commande_7:					; bit 1
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp commande_7
commande_7b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp commande_7b
	Sbic PIND,data			; On met  jour donnee en fonction de DATA
	Sbr donnee,0b00000010

commande_8:					; bit 0
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp commande_8

commande_8b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp commande_8b
	Sbic PIND,data			; On met  jour donnee en fonction de DATA
	Sbr donnee,0b00000001

commande_9:					; L'ack
	Sbic PIND,sicd			; on attend que SICD=0
	Rjmp commande_9
commande_9b:	
	Sbis PIND,sicd			; on attend que SICD=1
	Rjmp commande_9b

; Identification de la commande

	Cpi donnee,0x00			; commande 0h -> lecture
	Brne commande_10
	Rjmp lecture_0
commande_10:

	Cpi donnee,0x70			; commande 70h -> lecture
	Brne commande_11
	Rjmp lecture_0
commande_11:

	Cpi donnee,0xE0			; commande E0h -> lecture
	Brne commande_12
	Rjmp lecture_0
commande_12:

	Cpi donnee,0xF0			; commande F0h -> lecture
	Brne commande_13
	Rjmp lecture_0
commande_13:

	Cpi donnee,0xD1			; commande D1h -> hack
	Brne commande_14
	Rjmp hack_0
commande_14:


; Dans les autres cas c'est une criture

; **********************************************************************
; Ecriture de donne

; Pour chaque octet, on teste les 8 impulsions sur SICL pour viter les problmes de parasite sur SICD

; Le nb d'octet  rcuprer est gal au quartet de poids faible de la commande
	Mov octet,donnee
	Cbr octet,0xF0
	Dec octet

ecriture_0:
	Ldi temp,7				; On teste les 8 bits

ecriture_1:
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp ecriture_1
ecriture_2:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp ecriture_2

	Dec temp
	Brbc 2,ecriture_1

ecriture_3:					; L'ack
	Sbic PIND,sicd			; on attend que SICD=0
	Rjmp ecriture_3
ecriture_4:	
	Sbis PIND,sicd			; on attend que SICD=1
	Rjmp ecriture_4

	Dec octet
	Brbc 2,ecriture_0		; On fait n fois l'operation

; on a rcupr tous les octets, c'est fini

	Rjmp commande_0

; **********************************************************************
; Lecture de donnes

; On reste en lecture tant qu'il y a une lecture d'octet
; La premire criture correspond  une nouvelle commande
; L'criture est dcellable lors du premier bit

; Comme pour l'criture on teste tout les bits pour viter les problmes de parasite sur SICD

lecture_0:
	Clr donnee

lecture_1:
	Wdr						; Reset du watchdog
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp lecture_1
lecture_2:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp lecture_2	
	Sbic PIND,sicd			; Si SICD=0 -> lecture
	Rjmp commande_1c		; Sinon -> commande

	Ldi temp,5				; On teste les 7 bits restants

lecture_3:
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp lecture_3
lecture_4:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp lecture_4

	Dec temp
	Brbc 2,lecture_3

lecture_5:
	Sbis PIND,sicd			; on attend que SICD=1
	Rjmp lecture_5

	Rjmp lecture_1			; Octet suivant !

; **********************************************************************
; Hack : Test de dbut de lecture

; On rcupre la donne
hack_0:
	Cpi rehack,5
	Brne hack_0b
	Ldi rehack,6
	Rjmp hack2				; On hack la donnee entire !
hack_0b:
	Clr donneeb

hack_1:						; bit 7
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack_1
hack_1b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack_1b	
	Sbic PIND,data			; on met  jour donnee en fonction de DATA
	Sbr donneeb,0b10000000

hack_2:						; bit 6
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack_2
hack_2b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack_2b	
	Sbic PIND,data			; on met  jour donnee en fonction de DATA
	Sbr donneeb,0b01000000

hack_3:						; bit 5
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack_3
hack_3b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack_3b	
	Sbic PIND,data			; on met  jour donnee en fonction de DATA
	Sbr donneeb,0b00100000

hack_4:						; bit 4
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack_4
hack_4b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack_4b	
	Sbic PIND,data			; on met  jour donnee en fonction de DATA
	Sbr donneeb,0b00010000

hack_5:						; bit 3
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack_5
hack_5b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack_5b	
	Sbic PIND,data			; on met  jour donnee en fonction de DATA
	Sbr donneeb,0b00001000

hack_6:					; bit 2
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack_6
hack_6b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack_6b	
	Sbic PIND,data			; on met  jour donnee en fonction de DATA
	Sbr donneeb,0b00000100

hack_7:						; bit 1
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack_7
hack_7b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack_7b	
	Sbic PIND,data			; on met  jour donnee en fonction de DATA
	Sbr donneeb,0b00000010

hack_8:						; bit 0
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack_8
hack_8b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack_8b	
	Sbic PIND,data			; on met  jour donnee en fonction de DATA
	Sbr donneeb,0b00000001

hack_9:						; L'ack
	Sbic PIND,sicd			; on attend que SICD=0
	Rjmp hack_9
hack_9b:	
	Sbis PIND,sicd			; on attend que SICD=1
	Rjmp hack_9b
		
; Test de dbut de lecture		

	Cpi rehack,6			; Pas de rehack tant qu'on ne croise pas "D1h 00h"
	Brne rehack_0
	Cpi donneeb,0x00
	Brne rehack_6
	Cbi PORTB,0				; On eteint la led
	Clr rehack
	Rjmp commande_0
rehack_0:
	Cpi rehack,0
	Brne rehack_1
	Cpi donneeb,0x01
	Brne rehack_5
	Ldi rehack,1
	Rjmp commande_0
rehack_1:
	Cpi rehack,1
	Brne rehack_2
	Cpi donneeb,0x01
	Brne rehack_5
	Ldi rehack,4
	Rjmp commande_0
rehack_2:
	Cpi rehack,4
	Brne rehack_5
	Cpi donneeb,0x01
	Brne rehack_5
	Ldi rehack,5			; On rehack au prochain coup !
	Rjmp commande_0
rehack_5:
	Clr rehack
rehack_6:
	Rjmp commande_0

; **********************************************************************
; Hack : modification de la commande "D1h 01h" en "D1h 3Xh"
; On hack les 8 bits de donne

hack2:
	Mov donneeb,hack_dac

hack2_1:					; bit 7
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack2_1
	Sbi DDRD,data			; On force DATA
	Cbi PORTD,data
	Sbrc donneeb,7
	Sbi PORTD,data
hack2_1b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack2_1b

hack2_2:					; bit 6
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack2_2
	Cbi PORTD,data			; On force DATA
	Sbrc donneeb,6
	Sbi PORTD,data
hack2_2b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack2_2b

hack2_3:					; bit 5
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack2_3
	Cbi PORTD,data			; On force DATA
	Sbrc donneeb,5
	Sbi PORTD,data
hack2_3b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack2_3b

hack2_4:					; bit 4
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack2_4
	Cbi PORTD,data			; On force DATA
	Sbrc donneeb,4
	Sbi PORTD,data
hack2_4b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack2_4b

hack2_5:					; bit 3
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack2_5
	Cbi PORTD,data			; On force DATA
	Sbrc donneeb,3
	Sbi PORTD,data
hack2_5b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack2_5b

hack2_6:					; bit 2
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack2_6
	Cbi PORTD,data			; On force DATA
	Sbrc donneeb,2
	Sbi PORTD,data
hack2_6b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack2_6b	

hack2_7:					; bit 1
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack2_7
	Cbi PORTD,data			; On force DATA
	Sbrc donneeb,1
	Sbi PORTD,data
hack2_7b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack2_7b

hack2_8:					; bit 0
	Sbic PIND,sicl			; on attend que SICL=0
	Rjmp hack2_8
	Cbi PORTD,data			; On force DATA
	Sbrc donneeb,0
	Sbi PORTD,data
hack2_8b:
	Sbis PIND,sicl			; on attend que SICL=1
	Rjmp hack2_8b	

	Rcall tempo				; Tempo de 1s

	Cbi PORTD,data
	Cbi DDRD,data

hack2_9:					; L'ack
	Sbic PIND,sicd			; on attend que SICD=0
	Rjmp hack2_9
hack2_9b:	
	Sbis PIND,sicd			; on attend que SICD=1
	Rjmp hack2_9b

	Sbi PORTB,0				; On allume la led

	Rjmp commande_0

tempo :						; Tempo de 1s (=8 instructions)
	Nop
	Ret						; Rcall + Nop + Ret = 16 cycles !